package cz.muni.disa.adamiss.se;

import cz.muni.disa.adamiss.core.objects.TransactionFile;
import cz.muni.disa.adamiss.se.general.Dataset;
import cz.muni.disa.adamiss.se.stats.collectors.DatabaseStatCollector;
import cz.muni.disa.adamiss.se.stats.objects.DBReport;
import cz.muni.disa.adamiss.se.stats.objects.ExperimentReport;
import cz.muni.disa.adamiss.se.stats.runners.ExperimentRunner;
import cz.muni.disa.adamiss.se.structures.DatasetContainer;
import javafx.util.Pair;

import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import static cz.muni.disa.adamiss.se.loader.DatasetLoader.loadDataset;

public class Main {

    public static void main(String[] args) {

        List<Pair<Dataset,TransactionFile>> datasets = getDatasets();

        StringBuilder dbReportCSV = new StringBuilder("size,unique,avgLength,distribution").append(System.lineSeparator());
        List<DatasetContainer> containers = new ArrayList<>();
        for (var pair : datasets) {
            DBReport report = getDatabaseStats(pair.getValue());
            dbReportCSV.append(
                    report.toCSV()
            ).append(System.lineSeparator());

            containers.add(new DatasetContainer(pair.getKey(),pair.getValue(), report));
        }

        SimpleDateFormat formatter = new SimpleDateFormat();
        String outputFileDate = formatter.format(new Date());

        List<ExperimentReport> reports = ExperimentRunner.runAll(containers);
        StringBuilder eReportCSV = new StringBuilder("name,size,unique,avgLength,distribution,threshold,time,algorithm").append(System.lineSeparator());
        for (ExperimentReport erep:reports){
            eReportCSV.append(erep.toCSV()).append(System.lineSeparator());
        }
        saveAsCSV(eReportCSV.toString(), "Experiments" + outputFileDate);

        saveAsCSV(dbReportCSV.toString(), "DatasetStats" + outputFileDate);


    }

    private static List<Pair<Dataset,TransactionFile>> getDatasets() {
        List<Pair<Dataset,TransactionFile>> result = new ArrayList<>();
        for (Dataset dataset : Dataset.values()) {
            result.add(new Pair<>(dataset, loadDataset(dataset.path())));
        }
        return result;
    }

    public static DBReport getDatabaseStats(TransactionFile file) {
        DatabaseStatCollector collector = new DatabaseStatCollector();
        return collector.collect(file);
    }


    public static void saveAsCSV(String data, String name) {
        try {
            FileWriter report = new FileWriter(name + ".csv");
            report.write(data);
            report.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
